@@ -5,17 +5,11 @@ use Carp;
 use UNIVERSAL qw(isa);
 use Storable qw(dclone);  # For Deep Copy
 
-our $VERSION = '0.12';
-
-# Load YAML::Syck and, failing that, load YAML.
-eval {
-    require YAML::Syck;
-    YAML::Syck->import(qw(Load LoadFile));
-};
-if ($@) {
-    require YAML;
-    YAML->import(qw(Load LoadFile));
-}
+####################
+# Global Variables
+####################
+our $VERSION = '0.14';
+our @YAML_PREFS = qw(YAML::Syck YAML);
 
 #########################
 # Class Methods: Public
@@ -23,15 +17,26 @@ if ($@) {
 sub new {
     my ($class, %args) = @_;
     my $self = bless( \%args, ref($class) || $class );
-    $self->{stack} = [];  # For finding circular references.
 
-    # Load config from file, then load from a string.
-    $self->{config} = {};
-    $self->{config} = LoadFile( $self->{file} ) if exists $self->{file};
-    $self->{config} = Load( $self->{string} ) if exists $self->{string};
-    $self->{config} = dclone( $self->{object}->config ) 
-        if exists $self->{object};
+    # Load a YAML parser.
+    $self->{yaml_class} = $self->_load_yaml_class();
+
+    # Load config from file, string, or object.
+    if ( exists $self->{file} ) {
+        my $load_file = eval "\\&$self->{yaml_class}::LoadFile";
+        $self->{config} = $load_file->( $self->{file} );
+    } elsif ( exists $self->{string} ) {
+        my $load = eval "\\&$self->{yaml_class}::Load";
+        $self->{config} = $load->( $self->{string} );
+    } elsif ( exists $self->{object} ) {
+        $self->{config} = dclone( $self->{object}->{config} );
+    } else {
+        $self->{config} = {};
+    }
+
+    # Initialize internal state
     $self->_install_accessors();  # Install convenience accessors.
+    $self->{stack} = [];  # For finding circular references.
 
     return $self;
 }
@@ -128,17 +133,50 @@ sub _resolve_refs {
 sub _resolve_scalar {
     my ( $self, $value ) = @_;
     return unless defined $value;
-    my @parts = split /(\$(?:{\w+}|\w+))/, $value;
+    my @parts = grep length, # Empty strings are useless, discard them
+                     split /((?<!\\)\$(?:{\w+}|\w+))/, $value;
     for my $part (@parts) {
-        if ( $part =~ /^\$(?:{(\w+)}|(\w+))$/) {
+        if ( $part =~ /^(?<!\\)\$(?:{(\w+)}|(\w+))$/) {
             my $name = $1 || $2;
-            if ( exists $self->config->{$name} ) {
-                $part = $self->_get($name) unless ref $self->config->{$name};
-            }
+            $part = $self->_get($name) if exists $self->config->{$name};
+        } else {
+            $part =~ s/(\\*)\\(\$(?:{(\w+)}|(\w+)))/$1$2/g; # Unescape slashes
         }
     }
-    @parts = map { defined($_) ? $_ : "" } @parts;
-    return join "", @parts;
+    return $parts[0] if @parts == 1 and ref $parts[0]; # Preserve references
+    return join "", map { defined($_) ? $_ : "" } @parts;
+}
+
+# void _load_yaml_class
+#
+# Attempts to load a YAML class that can parse YAML for us.  We prefer the
+# yaml_class attribute over everything, then fall back to a previously loaded
+# YAML parser from @YAML_PREFS, and failing that try to load a parser from
+# @YAML_PREFS.
+sub _load_yaml_class {
+    my $self = shift;
+
+    # Always use what we were given.
+    if (defined $self->{yaml_class}) {
+        eval "require $self->{yaml_class}; 0;";
+        croak "$@\n" if $@;
+        return $self->{yaml_class};
+    }
+
+    # Use what's already been loaded.
+    for my $module (@YAML_PREFS) {
+        my $filename = $module . ".pm";
+        $filename =~ s{::}{/};
+        return $self->{yaml_class} = $module if exists $INC{$filename};
+    }
+
+    # Finally, try and load something.
+    for my $module (@YAML_PREFS) {
+        eval "require $module; 0;";
+        return $self->{yaml_class} = $module unless $@;
+    }
+
+    die "Could not load: " . join(" or ", @YAML_PREFS);
 }
 
 # void _install_accessors(void)
@@ -173,6 +211,7 @@ YAML::AppConfig - Manage configuration files with YAML and variable reference.
     ---
     etc_dir: /opt/etc
     foo_dir: $etc_dir/foo
+    bar_dir : ${foo_dir}bar
     some_array:
         - $foo_dir/place
     YAML
@@ -193,50 +232,61 @@ YAML::AppConfig - Manage configuration files with YAML and variable reference.
     # Set etc_dir in three different ways, all equivalent.
     $conf->set("etc_dir", "/usr/local/etc");
     $conf->set_etc_dir("/usr/local/etc");
-    $conf->config->{etc_dr} = "/usr/local/etc";
+    $conf->config->{etc_dir} = "/usr/local/etc";
 
-    # Notice that when variables change that that affects other variables:
+    # Notice that changed variables affect other variables:
     $config->get_foo_dir;          # now returns /usr/local/etc/foo
     $config->get_some_array->[0];  # returns /usr/local/etc/foo/place
 
+    # You can escape variables for concatenation purposes:
+    $config->get_bar_dir;  # Returns '/usr/local/etc/foobar'
+
 =head1 DESCRIPTION
 
 YAML::AppConfig extends the work done in L<Config::YAML> and
 L<YAML::ConfigFile> to allow variable reference between settings.  Essentialy
 your configuration file is a hash serialized to YAML.  Scalar values that have
-$foo_var type values in them will have interpolation done on them.  If
-$foo_var is a key in the configuration file it will be substituted, otherwise
-it will be left alone.  $foo_var must be a reference to a scalar value and not
-a hash or array, otherwise it won't be interpolated.
+$foo_var type values in them will have interpolation done on them.  If $foo_var
+is a key in the configuration file it will be substituted, otherwise it will be
+left alone.  $foo_var must be a reference to a scalar value and not a hash or
+array, otherwise it won't be interpolated.
 
-Either L<YAML> or L<YAML::Syck> is used underneath.  If L<YAML::Syck> is found
-it will be used over L<YAML>.
+Either L<YAML> or L<YAML::Syck> is used underneath.  You can also specify your
+own YAML parser by using the C<yaml_class> attribute to C<new()>.  By default
+the value of the C<yaml_class> attribute is preferred over everything.  Failing
+that, we check to see if C<YAML::Syck> or C<YAML> is loaded, and if so use that
+(prefering Syck).  If nothing is loaded and the C<yaml_class> attribute was not
+given then we try to load a YAML parser, starting with C<YAML::Syck> and then
+trying C<YAML>.  If this is Too AI for you and bites your ass then you can
+force behavior using C<yaml_class>, which is really why it exists.
 
 =head1 USING VARIABLES
 
+Variable names refer to items at the top of the YAML configuration.  There is
+currently no way to refer to items nested inside the configuration.  If a
+variable is not known, because it doesn't match the name of a top level
+configuration key, then no substitution is done on it and it is left verbatim
+in the value.
+
 Variables names must match one of C</\$\w+/> or C</\${\w+}/>.  Just like in
 Perl the C<${foo}> form is to let you have values of the form C<${foo}bar> and
 have the variable be treated as C<$foo> instead of C<$foobar>.  
 
-If a variable is not recognized it will be left as is in the value, it won't be
-interpolated away or cause a warning.  Unknown variables are not recognized, as
-are variables that refer to references (e.g. hashes or arrays).  Currently
-variables can only address items in the top-most level of the YAML
-configuration file (i.e. the top most level of the hash the conf file
-represents).
+You can escape a variable by using a backslash before the dollar sign.  For
+example C<\$foo> will result in a literal C<$foo> being used, and thus no
+interpolation will be done on it.  If you should want a literal C<\$foo> then
+use two slashes, C<\\$foo>.  Should you want a literal C<\\$foo> then use three
+slashes, and so on.  Escaping can be used with C<${foo}> style variables too.
 
-There is currently no way to escape a variable.  For simple cases this is not a
-problem because unrecongnized variables are left as is.  However, if you have a
-setting named C<foo> in the top of your YAML file and you want to use a literal
-value of C<'$foo'> then you are, in a way, out of luck.  As a work around, you
-can access the raw values from the Perl side by passing in C<$no_resolve> to
-C<get()>.  I realize this is not ideal and there are still cases were you are
-SOL, but I haven't hit this problem yet and so I have not been inclined to
-solve it.
+Variables can be references to more complex data structures.  So it's possible
+to define a list and assign it to a top level configuration key and then reuse
+that list anywhere else in the configuration file.  Just like in Perl, if a
+variable refering to a reference is used in a string the raw memory address
+will be its value, so be warned.
 
 =head1 METHODS
 
-=head2 new
+=head2 new(%args)
 
 Creates a new YAML::AppConfig object and returns it.  new() accepts the
 following key values pairs:
@@ -259,6 +309,13 @@ A L<YAML::AppConfig> object which will be deep copied into your object.
 
 If true no attempt at variable resolution is done on calls to C<get()>.
 
+=item yaml_class
+
+The name of the class we should use to find our C<LoadFile> and C<Load>
+functions for parsing YAML files and strings, respectively.  The named class
+should provide both C<LoadFile> and C<Load> as functions and should be loadable
+via C<require>.
+
 =back
 
 =head2 get(key, [no_resolve])
@@ -271,7 +328,7 @@ returned, no variable interpolation is done.
 
 Similar to C<get()> except you can also provide a value for the setting.
 
-=head2 get_*
+=head2 get_*([no_resolve])
 
 Convenience methods to retrieve values using a method, see C<get>.  For
 example if foo_bar is a configuration value in your YAML file then
@@ -279,7 +336,7 @@ C<get_foo_bar> retrieves its value.  These methods are curried versions of
 C<get>.  These functions all take a single optional argument, C<$no_resolve>,
 which is the same as C<get()'s> C<$no_resolve>.
 
-=head2 set_*
+=head2 set_*(value)
 
 A convience method to set values using a method, see C<set> and C<get_*>.
 These methods are curried versions of C<set>.
@@ -293,13 +350,19 @@ interpolated, this is just the raw data.
 
 Returns the keys in C<config()> sorted from first to last.
 
+=head2 merge(%args)
+
+Merge takes another YAML configuration and merges it into this one.  C<%args>
+are the same as those passed to C<new()>, so the configuration can come from a
+file, string, or existing L<YAML::AppConfig> object.
+
 =head1 AUTHORS
 
+Matthew O'Connor E<lt>matthew@canonical.orgE<gt>
+
 Original implementations by Kirrily "Skud" Robert (as L<YAML::ConfigFile>) and
 Shawn Boyette (as L<Config::YAML>).
 
-Matthew O'Connor E<lt>matthew@canonical.orgE<gt>
-
 =head1 SEE ALSO
 
 L<YAML>, L<YAML::Syck>, L<Config::YAML>, L<YAML::ConfigFile>
